/********************************************************************

  XPP is a general purpose expression parser. It has fixed rules for how
  numbers and identifiers look, but all other aspects of it can be configured.

  --- Initialization ---

  xpp_t *xpp_new();

  Creates a parser object. You can't parse anything with it yet, you need
  to define some operators before you can use it.

  --- Numbers ---

  Numbers can be written only in decimal base. Negative numbers are not
  recognized. Instead, you can introduce a negation operator. Numbers come
  in two formats: integer and floating point. Integer numbers are simply
  strings of digits 0-9. A leading 0 doesn't have any special meaning.

  Floating point numbers consist of three parts:

    int . frac exp

  int and frac are regular integers. The exponent part starts with an
  E or e, followed by an option sign - or +, and then followed by another
  integer. Both frac and exp are optional.

  --- Identifiers ---

  Identifiers are strings of letters, followed by strings of letters or
  digits. By default, the underscore character is considered a letter
  but if you define an operator which starts with an underscore, then
  it's classified as a non-letter from there on. 

  --- Operators ---

  Operators can be defined by the user, by using the following functions:

  void xpp_binary_operator  (xpp_t* p,char *str,int disp,int prec, int assoc);
  void xpp_prefix_operator  (xpp_t* p,char *str,int disp);
  void xpp_postfix_operator (xpp_t *p,char *str,int disp);

  The number 'disp' is the display number, which is discussed later in this
  document.

  In each of these functions, the first character of the given string is
  considered to be an 'operator character' from here on. You can no longer
  use that character as part of an identifier.

  Postfix operators have the greatest precedence. Prefix operators come next,
  followed by binary operators. Associativity for binary operators can be
  given as 'L' or 'R'.

  --- Parenthesis ---

  By default, XPP doesn't have any rules regarding how to parse parenthesised
  expressions. You can introduce your rules using the following functions:

  void xpp_parenthesis         (xpp_t *p,char *left,char *right,int disp);
  void xpp_postfix_parenthesis (xpp_t *p,char *left,char *right,int disp);

  left is the opening parenthesis and right is the closing parenthesis.
  The second function introduces an operator parenthesis, like a function
  call or array access. A parenthesis pair can be either a normal 
  parenthesised expression, a binary operator on the expression to the left
  and the expression inside, or both.

  The opening and closing strings don't have to match visually or in any
  other way. You can have pairs such as:

    .{  }
    ((  ]

  etc.

  --- Quoted Strings ---

  Multiple flavors of quoted strings can be defined using the function:

  void xpp_quote(xpp_t *p, char *left, char *right, int escape, int disp);

  Again, left and right are the quote opening and closing strings. The 
  escape char is used for escaping special characters. The recognized
  escape sequences are:

   {ESCAPE}n     -> newline
   {ESCAPE}t     -> tab
   {ESCAPE}xHH   -> the byte corresponding to hexadecimal value HH
   {ESCAPE}<any> -> the byte immediately following the escape character

  Each quoting mark can have a different display value.

  --- Running ---

  You run the parser using:

  typedef struct expr
  {
    int op;
    struct expr *left, *right;
    void *data;
    int pos;
  } expr_t;

  expr_t* xpp_run(xpp_t *p,char *input);

  The returned expression object will define a parse tree for your input.
  Each node in the tree has an 'op' field. This field identifies which
  operator the node corresponds to. The values for op fields are assigned
  during configuration, when the corresponding operators were created. The
  'display numbers' in the above configuration functions. For instance,
  if we have:
 
   xpp_binary_operator(p,"+",OP_ADD,50,'L');
   E= xpp_run(p, "3+4");

  E->op will be OP_ADD.

  There are five predefined op values:

   XPP_INT, XPP_FLO, XPP_IDENT, XPP_ERROR, XPP_LAST 

  XPP_LAST is never used in anything, it's just there to make it easier to
  define your own enumeration for your operators etc, like this:

  enum myops { OP_ADD= XPP_LAST, OP_SUB, OP_MUL, ... }

  For the other predefined op values, E->data contains the corresponding 
  string. For the above example:

   E->left->op   ==  XPP_INT
   E->left->data ==  "3"

  For all nodes, the 'pos' field tells you at which point in the input 
  this node occured.  For instance, 

   E->pos       == 1 
   E->left->pos == 0

  for our simple example.

  --- Errors ---

  If there is an error during parsing, an error expression is returned.
  The data field in the expression is an error message. As with the
  other kinds of nodes, E->pos gives you the position of the character
  which caused the error.
  
  After an error, the parser is still usable. An error makes the parser
  reset to its state before xpp_run.

  --- Shutting Down ---

  When you're done with it, you can free the parser using:

  void xpp_free(xpp_t *p);

  If you have some expressions you no longer want, you can free them with
  this:

  void xpp_free_expr(expr_t *expr);

  Expressions are completely independent of the parsers and can be freed
  at any time.

  Note that the 'data' fields in expressions are copies from the original
  input string. So, the expressions are independent also of the input string.

 ********************************************************************/
#ifndef _XPP_PROTO
#define _XPP_PROTO
#endif

typedef struct xpp xpp_t;

typedef struct expr
{
  int op;
  struct expr *left, *right;
  void *data;
  int pos;
} expr_t;

enum { XPP_INT, XPP_FLO, XPP_IDENT, XPP_ERROR, XPP_LAST };


_XPP_PROTO xpp_t *xpp_new();
_XPP_PROTO void xpp_binary_operator
      (xpp_t* p,char *str,int disp,int prec, int assoc);
_XPP_PROTO void xpp_prefix_operator
      (xpp_t* p,char *str,int disp);
_XPP_PROTO void xpp_postfix_operator
      (xpp_t *p,char *str,int disp);
_XPP_PROTO void xpp_parenthesis
      (xpp_t *p,char *left,char *right,int disp);
_XPP_PROTO void xpp_postfix_parenthesis
      (xpp_t *p,char *left,char *right,int disp);
_XPP_PROTO void xpp_quote
      (xpp_t *p, char *left, char *right, int escape, int disp);
_XPP_PROTO expr_t* xpp_run(xpp_t *p,char *input);
_XPP_PROTO void xpp_free_expr(expr_t *expr);
_XPP_PROTO void xpp_free(xpp_t *p);
